@@ -92,7 +92,7 @@ do_node_status(void)
 
 	/* Check node exists and is really a standby */
 
-	if (get_node_record(conn, config_file_options.node_id, &node_info) != RECORD_FOUND)
+	if (get_node_record_with_upstream(conn, config_file_options.node_id, &node_info) != RECORD_FOUND)
 	{
 		log_error(_("no record found for node %i"), config_file_options.node_id);
 		PQfinish(conn);
@@ -308,14 +308,16 @@ do_node_status(void)
 
 	/*
 	 * check for missing replication slots - we do this regardless of
-	 * what "max_replication_slots" is set to
+	 * what "max_replication_slots" is set to, in case the downstream
+	 * node was configured with "use_replication_slots=true" and is
+	 * expecting a replication slot to be available
 	 */
 
 	{
 		NodeInfoList missing_slots = T_NODE_INFO_LIST_INITIALIZER;
-		get_downsteam_nodes_with_missing_slot(conn,
-											  config_file_options.node_id,
-											  &missing_slots);
+		get_downstream_nodes_with_missing_slot(conn,
+											   config_file_options.node_id,
+											   &missing_slots);
 
 		if (missing_slots.node_count > 0)
 		{
@@ -1814,7 +1816,7 @@ do_node_rejoin(void)
 	 * Forcibly rewind node if requested (this is mainly for use when this
 	 * action is being executed by "repmgr standby switchover")
 	 */
-	if (runtime_options.force_rewind == true)
+	if (runtime_options.force_rewind == true && runtime_options.dry_run == false)
 	{
 		int			ret;
 		PQExpBufferData		filebuf;
@@ -1949,6 +1951,12 @@ do_node_rejoin(void)
 		}
 	}
 
+	if (runtime_options.dry_run == true)
+	{
+		log_info(_("prerequisites for executing NODE REJOIN are met"));
+		exit(SUCCESS);
+	}
+
 	initPQExpBuffer(&follow_output);
 
 	success = do_standby_follow_internal(upstream_conn,
