@@ -176,7 +176,7 @@ main(int argc, char **argv)
 		strncpy(runtime_options.username, pw->pw_name, MAXLEN);
 	}
 
-	while ((c = getopt_long(argc, argv, "?Vb:f:FWd:h:p:U:R:S:D:ckL:tvC:", long_options,
+	while ((c = getopt_long(argc, argv, "?Vb:f:FWd:h:p:U:R:S:D:ck:L:tvC:", long_options,
 							&optindex)) != -1)
 	{
 		/*
@@ -388,6 +388,11 @@ main(int argc, char **argv)
 				runtime_options.without_barman = true;
 				break;
 
+			case OPT_RECOVERY_CONF_ONLY:
+				runtime_options.recovery_conf_only = true;
+				break;
+
+
 				/*---------------------------
 				 * "standby register" options
 				 *---------------------------
@@ -993,31 +998,10 @@ main(int argc, char **argv)
 	}
 
 
-
-	/* check for conflicts between runtime options and configuration file */
-	/* ================================================================== */
-
-	if (action == STANDBY_CLONE)
-	{
-		standy_clone_mode mode = get_standby_clone_mode();
-
-		if (mode == barman && runtime_options.without_barman == false
-			&& config_file_options.use_replication_slots == true)
-		{
-			log_error(_("STANDBY CLONE in Barman mode is incompatible with configuration option \"use_replication_slots\""));
-			log_hint(_("set \"use_replication_slots\" to \"no\" in repmgr.conf, or use --without-barman to clone directly from the upstream server"));
-			exit(ERR_BAD_CONFIG);
-		}
-	}
-
-
 	/*
 	 * Check for configuration file items which can be overriden by runtime
 	 * options
-	 */
-
-	/*
-	 * ============================================================================
+	 * =====================================================================
 	 */
 
 	/*
@@ -1495,6 +1479,39 @@ check_cli_parameters(const int action)
 		}
 	}
 
+	if (runtime_options.replication_user[0])
+	{
+		switch (action)
+		{
+			case PRIMARY_REGISTER:
+			case STANDBY_REGISTER:
+			case STANDBY_CLONE:
+				break;
+
+			case STANDBY_FOLLOW:
+				item_list_append_format(&cli_warnings,
+										_("--replication-user ignored when executing %s"),
+										action_name(action));
+			default:
+				item_list_append_format(&cli_warnings,
+										_("--replication-user not required when executing %s"),
+										action_name(action));
+		}
+	}
+
+	if (runtime_options.recovery_conf_only == true)
+	{
+		switch (action)
+		{
+			case STANDBY_CLONE:
+				break;
+			default:
+				item_list_append_format(&cli_warnings,
+										_("--create-recovery-conf will be ignored when executing %s"),
+										action_name(action));
+		}
+	}
+
 	if (runtime_options.event[0])
 	{
 		switch (action)
@@ -1508,25 +1525,6 @@ check_cli_parameters(const int action)
 		}
 	}
 
-	if (runtime_options.replication_user[0])
-	{
-		switch (action)
-		{
-			case PRIMARY_REGISTER:
-			case STANDBY_REGISTER:
-				break;
-			case STANDBY_CLONE:
-			case STANDBY_FOLLOW:
-				item_list_append_format(&cli_warnings,
-										_("--replication-user ignored when executing %s)"),
-										action_name(action));
-			default:
-				item_list_append_format(&cli_warnings,
-										_("--replication-user not required when executing %s"),
-										action_name(action));
-		}
-	}
-
 	if (runtime_options.limit_provided)
 	{
 		switch (action)
@@ -1777,10 +1775,11 @@ do_help(void)
 
 	printf(_("Usage:\n"));
 	printf(_("    %s [OPTIONS] primary {register|unregister}\n"), progname());
-	printf(_("    %s [OPTIONS] standby {register|unregister|clone|promote|follow}\n"), progname());
+	printf(_("    %s [OPTIONS] standby {register|unregister|clone|promote|follow|switchover}\n"), progname());
 	printf(_("    %s [OPTIONS] bdr     {register|unregister}\n"), progname());
-	printf(_("    %s [OPTIONS] node    status\n"), progname());
+	printf(_("    %s [OPTIONS] node    {status|check|rejoin|service}\n"), progname());
 	printf(_("    %s [OPTIONS] cluster {show|event|matrix|crosscheck}\n"), progname());
+	printf(_("    %s [OPTIONS] witness {register|unregister}\n"), progname());
 
 	puts("");
 
@@ -2151,10 +2150,19 @@ local_command(const char *command, PQExpBufferData *outputbuf)
 }
 
 
+/*
+ * get_superuser_connection()
+ *
+ * Check if provided connection "conn" is a superuser connection, if not attempt to
+ * make a superuser connection "superuser_conn" with the provided --superuser parameter.
+ *
+ * "privileged_conn" is set to whichever connection is the superuser connection.
+ */
 void
 get_superuser_connection(PGconn **conn, PGconn **superuser_conn, PGconn **privileged_conn)
 {
 	t_connection_user userinfo = T_CONNECTION_USER_INITIALIZER;
+	t_conninfo_param_list conninfo_params = T_CONNINFO_PARAM_LIST_INITIALIZER;
 	bool		is_superuser = false;
 
 	/* this should never happen */
@@ -2180,9 +2188,11 @@ get_superuser_connection(PGconn **conn, PGconn **superuser_conn, PGconn **privil
 		exit(ERR_BAD_CONFIG);
 	}
 
-	*superuser_conn = establish_db_connection_as_user(config_file_options.conninfo,
-													  runtime_options.superuser,
-													  false);
+	initialize_conninfo_params(&conninfo_params, false);
+	conn_to_param_list(*conn, &conninfo_params);
+	param_set(&conninfo_params, "user", runtime_options.superuser);
+
+	*superuser_conn = establish_db_connection_by_params(&conninfo_params, false);
 
 	if (PQstatus(*superuser_conn) != CONNECTION_OK)
 	{
@@ -2707,7 +2717,7 @@ init_node_record(t_node_info *node_record)
 
 	if (config_file_options.replication_user[0] != '\0')
 	{
-		/* replication user explicitly provided */
+		/* replication user explicitly provided in configuration file */
 		strncpy(node_record->repluser, config_file_options.replication_user, NAMEDATALEN);
 	}
 	else
