@@ -85,6 +85,36 @@ static void update_monitoring_history(void);
 static const char * format_failover_state(FailoverState failover_state);
 
 
+void
+handle_sigint_physical(SIGNAL_ARGS)
+{
+	PGconn *writeable_conn;
+	PQExpBufferData event_details;
+
+	initPQExpBuffer(&event_details);
+
+	appendPQExpBuffer(&event_details,
+					  "%s signal received",
+					  postgres_signal_arg == SIGTERM
+					  ? "TERM" : "INT");
+
+	if (local_node_info.type == PRIMARY)
+		writeable_conn = local_conn;
+	else
+		writeable_conn = primary_conn;
+
+	create_event_notification(writeable_conn,
+							  &config_file_options,
+							  config_file_options.node_id,
+							  "repmgrd_shutdown",
+							  true,
+							  event_details.data);
+
+	termPQExpBuffer(&event_details);
+
+	terminate(SUCCESS);
+}
+
 /* perform some sanity checks on the node's configuration */
 
 void
@@ -113,6 +143,14 @@ do_physical_node_check(void)
 				log_error(_("this node is marked as inactive and cannot be used as a failover target"));
 				log_hint(_("%s"), hint);
 				PQfinish(local_conn);
+
+				create_event_notification(NULL,
+										  &config_file_options,
+										  config_file_options.node_id,
+										  "repmgrd_shutdown",
+										  false,
+										  "node is inactive and cannot be used as a failover target");
+
 				terminate(ERR_BAD_CONFIG);
 
 			case FAILOVER_MANUAL:
@@ -280,6 +318,7 @@ monitor_streaming_primary(void)
 
 				monitoring_state = MS_DEGRADED;
 				INSTR_TIME_SET_CURRENT(degraded_monitoring_start);
+				log_notice(_("unable to connect to local node, falling back to degraded monitoring"));
 			}
 
 		}
@@ -303,7 +342,7 @@ monitor_streaming_primary(void)
 				create_event_notification(NULL,
 										  &config_file_options,
 										  config_file_options.node_id,
-										  "repmgrd_terminate",
+										  "repmgrd_shutdown",
 										  true,
 										  event_details.data);
 
@@ -325,22 +364,20 @@ monitor_streaming_primary(void)
 				else
 				{
 					local_node_info.node_status = NODE_STATUS_UP;
-					monitoring_state = MS_NORMAL;
-
-					initPQExpBuffer(&event_details);
 
 					/* check to see if the node has been restored as a standby */
 					if (get_recovery_type(local_conn) == RECTYPE_STANDBY)
 					{
 						PGconn *new_primary_conn;
 
+						initPQExpBuffer(&event_details);
+
 						appendPQExpBuffer(&event_details,
 										  _("reconnected to node after %i seconds, node is now a standby, switching to standby monitoring"),
 										  degraded_monitoring_elapsed);
 						log_notice("%s", event_details.data);
 						termPQExpBuffer(&event_details);
 
-
 						primary_node_id = UNKNOWN_NODE_ID;
 
 						new_primary_conn = get_primary_connection_quiet(local_conn, &primary_node_id, NULL);
@@ -353,54 +390,103 @@ monitor_streaming_primary(void)
 						else
 						{
 							RecordStatus record_status;
-							int i = 0;
 
 							log_debug("primary node id is now %i", primary_node_id);
 
-							/*
-							 * poll for a while until record type is returned as "STANDBY" - it's possible
-							 * that there's a gap between the server being restarted and the record
-							 * being updated
-							 */
-							for (i = 0; i < 30; i++)
-							{
-								/*
-								 * try and refresh the local node record from the primary, as the updated
-								 * local node record may not have been replicated yet
-								 */
-
-								record_status = get_node_record(new_primary_conn, config_file_options.node_id, &local_node_info);
-
-								if (record_status == RECORD_FOUND)
-								{
-									log_debug("type = %s", get_node_type_string(local_node_info.type));
-
-									if (local_node_info.type == STANDBY)
-									{
-										PQfinish(new_primary_conn);
-
-										/* XXX add event notification */
-										return;
-									}
-								}
-								sleep(1);
-							}
-
-							PQfinish(new_primary_conn);
+							record_status = get_node_record(new_primary_conn, config_file_options.node_id, &local_node_info);
 
 							if (record_status == RECORD_FOUND)
 							{
-								log_warning(_("repmgr node record is still %s"), get_node_type_string(local_node_info.type));
+								bool resume_monitoring = true;
+
+								log_debug("node %i is registered with type = %s",
+										  config_file_options.node_id,
+										  get_node_type_string(local_node_info.type));
+
+								/*
+								 * node has recovered but metadata not updated - we can do that ourselves,
+								 */
+								if (local_node_info.type == PRIMARY)
+								{
+									log_notice(_("node \"%s\" (ID: %i) still registered as primary, setting to standby"),
+											   config_file_options.node_name,
+											   config_file_options.node_id);
+
+									if (update_node_record_set_active_standby(new_primary_conn, config_file_options.node_id) == false)
+									{
+										resume_monitoring = false;
+									}
+									else
+									{
+										record_status = get_node_record(new_primary_conn, config_file_options.node_id, &local_node_info);
+
+										if (record_status != RECORD_FOUND)
+										{
+											resume_monitoring = false;
+										}
+									}
+								}
+
+								if (resume_monitoring == true)
+								{
+									monitoring_state = MS_NORMAL;
+									log_notice(_("former primary has been restored as standby after %i seconds, updating node record and resuming monitoring"),
+											   degraded_monitoring_elapsed);
+
+									initPQExpBuffer(&event_details);
+
+									appendPQExpBuffer(&event_details,
+													  _("node restored as standby after %i seconds, monitoring connection to upstream node %i"),
+													  degraded_monitoring_elapsed,
+													  local_node_info.upstream_node_id);
+
+									create_event_notification(new_primary_conn,
+															  &config_file_options,
+															  config_file_options.node_id,
+															  "repmgrd_standby_reconnect",
+															  true,
+															  event_details.data);
+
+
+									termPQExpBuffer(&event_details);
+
+									PQfinish(new_primary_conn);
+
+									/* restart monitoring as standby */
+									return;
+								}
 							}
-							else
+							else if (record_status == RECORD_NOT_FOUND)
 							{
-								log_error(_("no metadata record found for this node"));
+								PQExpBufferData event_details;
+								initPQExpBuffer(&event_details);
+
+								appendPQExpBuffer(&event_details,
+												  _("no metadata record found for this node on current primary %i"),
+												  primary_node_id);
+
+								log_error("%s", event_details.data);
 								log_hint(_("check that 'repmgr (primary|standby) register' was executed for this node"));
+
+								PQfinish(new_primary_conn);
+
+								create_event_notification(NULL,
+														  &config_file_options,
+														  config_file_options.node_id,
+														  "repmgrd_shutdown",
+														  false,
+														  event_details.data);
+								termPQExpBuffer(&event_details);
+
+								terminate(ERR_BAD_CONFIG);
 							}
 						}
 					}
 					else
 					{
+						monitoring_state = MS_NORMAL;
+
+						initPQExpBuffer(&event_details);
 						appendPQExpBuffer(&event_details,
 										  _("reconnected to primary node after %i seconds, resuming monitoring"),
 										  degraded_monitoring_elapsed);
@@ -441,7 +527,7 @@ loop:
 
 				if (monitoring_state == MS_DEGRADED)
 				{
-					log_detail(_("waiting for primary to reappear"));
+					log_detail(_("waiting for the node to become available"));
 				}
 
 				INSTR_TIME_SET_CURRENT(log_status_interval_start);
@@ -472,8 +558,6 @@ loop:
 			got_SIGHUP = false;
 		}
 
-
-
 		log_verbose(LOG_DEBUG, "sleeping %i seconds (parameter \"monitor_interval_secs\")",
 					config_file_options.monitor_interval_secs);
 
@@ -1009,6 +1093,9 @@ loop:
 			got_SIGHUP = false;
 		}
 
+		log_verbose(LOG_DEBUG, "sleeping %i seconds (parameter \"monitor_interval_secs\")",
+					config_file_options.monitor_interval_secs);
+
 		sleep(config_file_options.monitor_interval_secs);
 	}
 }
@@ -1029,9 +1116,26 @@ monitor_streaming_witness(void)
 
 	if (get_primary_node_record(local_conn, &upstream_node_info) == false)
 	{
-		log_error(_("unable to retrieve record for primary node"));
+		PQExpBufferData event_details;
+
+		initPQExpBuffer(&event_details);
+
+		appendPQExpBuffer(&event_details,
+						  _("unable to retrieve record for primary node"));
+
+		log_error("%s", event_details.data);
 		log_hint(_("execute \"repmgr witness register --force\" to update the witness node "));
 		PQfinish(local_conn);
+
+		create_event_notification(NULL,
+								  &config_file_options,
+								  config_file_options.node_id,
+								  "repmgrd_shutdown",
+								  false,
+								  event_details.data);
+
+		termPQExpBuffer(&event_details);
+
 		terminate(ERR_BAD_CONFIG);
 	}
 
@@ -1339,6 +1443,9 @@ loop:
 			got_SIGHUP = false;
 		}
 
+		log_verbose(LOG_DEBUG, "sleeping %i seconds (parameter \"monitor_interval_secs\")",
+					config_file_options.monitor_interval_secs);
+
 		sleep(config_file_options.monitor_interval_secs);
 	}
 
@@ -1376,7 +1483,7 @@ do_primary_failover(void)
 
 		failover_state = promote_self();
 	}
-	else if (election_result == ELECTION_LOST)
+	else if (election_result == ELECTION_LOST || election_result == ELECTION_NOT_CANDIDATE)
 	{
 		log_info(_("follower node awaiting notification from the candidate node"));
 		failover_state = FAILOVER_STATE_WAITING_NEW_PRIMARY;
@@ -1392,7 +1499,7 @@ do_primary_failover(void)
 
 		/* TODO: rerun election if new primary doesn't appear after timeout */
 
-		/* either follow or time out; either way resume monitoring */
+		/* either follow, self-promote or time out; either way resume monitoring */
 		if (wait_primary_notification(&new_primary_id) == true)
 		{
 			/* if primary has reappeared, no action needed */
@@ -1443,11 +1550,9 @@ do_primary_failover(void)
 											  &config_file_options,
 											  local_node_info.node_id,
 											  "standby_disconnect_manual",
-
-					/*
-					 * here "true" indicates the action has occurred as
-					 * expected
-					 */
+											  /*
+											   * here "true" indicates the action has occurred as expected
+											   */
 											  true,
 											  event_details.data);
 					PQfinish(new_primary_conn);
@@ -1666,7 +1771,7 @@ do_upstream_standby_failover(void)
 	t_node_info primary_node_info = T_NODE_INFO_INITIALIZER;
 	RecordStatus record_status = RECORD_NOT_FOUND;
 	RecoveryType primary_type = RECTYPE_UNKNOWN;
-	int			r;
+	int			i, r;
 	char		parsed_follow_command[MAXPGPATH] = "";
 
 	PQfinish(upstream_conn);
@@ -1751,8 +1856,30 @@ do_upstream_standby_failover(void)
 		termPQExpBuffer(&event_details);
 	}
 
-	/* reconnect to local node */
-	local_conn = establish_db_connection(config_file_options.conninfo, false);
+	/*
+	 * It's possible that the standby is still starting up after the "follow_command"
+	 * completes, so poll for a while until we get a connection.
+	 */
+
+	for (i = 0; i < config_file_options.standby_reconnect_timeout; i++)
+	{
+		local_conn = establish_db_connection(local_node_info.conninfo, false);
+
+		if (PQstatus(local_conn) == CONNECTION_OK)
+			break;
+
+		log_debug("sleeping 1 second; %i of %i attempts to reconnect to local node",
+				  i + 1,
+				  config_file_options.standby_reconnect_timeout);
+		sleep(1);
+	}
+
+	if (PQstatus(local_conn) != CONNECTION_OK)
+	{
+		log_error(_("unable to reconnect to local node %i"),
+				  local_node_info.node_id);
+		return FAILOVER_STATE_FOLLOW_FAIL;
+	}
 
 	/* refresh shared memory settings which will have been zapped by the restart */
 	repmgrd_set_local_node_id(local_conn, config_file_options.node_id);
@@ -2032,7 +2159,7 @@ follow_new_primary(int new_primary_id)
 	char		parsed_follow_command[MAXPGPATH] = "";
 
 	PQExpBufferData event_details;
-	int			r;
+	int			i, r;
 
 	/* Store details of the failed node here */
 	t_node_info failed_primary = T_NODE_INFO_INITIALIZER;
@@ -2144,8 +2271,6 @@ follow_new_primary(int new_primary_id)
 		return FAILOVER_STATE_FOLLOW_FAIL;
 	}
 
-
-
 	/*
 	 * refresh local copy of local and primary node records - we get these
 	 * directly from the primary to ensure they're the current version
@@ -2168,7 +2293,30 @@ follow_new_primary(int new_primary_id)
 		return FAILOVER_STATE_FOLLOW_FAIL;
 	}
 
-	local_conn = establish_db_connection(local_node_info.conninfo, false);
+	/*
+	 * It's possible that the standby is still starting up after the "follow_command"
+	 * completes, so poll for a while until we get a connection.
+	 */
+
+	for (i = 0; i < config_file_options.standby_reconnect_timeout; i++)
+	{
+		local_conn = establish_db_connection(local_node_info.conninfo, false);
+
+		if (PQstatus(local_conn) == CONNECTION_OK)
+			break;
+
+		log_debug("sleeping 1 second; %i of %i attempts to reconnect to local node",
+				  i + 1,
+				  config_file_options.standby_reconnect_timeout);
+		sleep(1);
+	}
+
+	if (PQstatus(local_conn) != CONNECTION_OK)
+	{
+		log_error(_("unable to reconnect to local node %i"),
+				  local_node_info.node_id);
+		return FAILOVER_STATE_FOLLOW_FAIL;
+	}
 
 	/* refresh shared memory settings which will have been zapped by the restart */
 	repmgrd_set_local_node_id(local_conn, config_file_options.node_id);
@@ -2181,8 +2329,7 @@ follow_new_primary(int new_primary_id)
 
 	log_notice("%s", event_details.data);
 
-	create_event_notification(
-							  upstream_conn,
+	create_event_notification(upstream_conn,
 							  &config_file_options,
 							  local_node_info.node_id,
 							  "repmgrd_failover_follow",
@@ -2356,18 +2503,20 @@ do_election(void)
 
 	if (config_file_options.failover == FAILOVER_MANUAL)
 	{
-		log_notice(_("this node is not configured for automatic failover so will not be considered as promotion candidate"));
+		log_notice(_("this node is not configured for automatic failover so will not be considered as promotion candidate, and will not follow the new primary"));
+		log_detail(_("\"failover\" is set to \"manual\" in repmgr.conf"));
+		log_hint(_("manually execute \"repmgr standby follow\" to have this node follow the new primary"));
 
-		return ELECTION_LOST;
+		return ELECTION_NOT_CANDIDATE;
 	}
 
-	/* node priority is set to zero - don't ever become a candidate */
+	/* node priority is set to zero - don't become a candidate, and lose by default */
 	if (local_node_info.priority <= 0)
 	{
 		log_notice(_("this node's priority is %i so will not be considered as an automatic promotion candidate"),
 				   local_node_info.priority);
 
-		return ELECTION_NOT_CANDIDATE;
+		return ELECTION_LOST;
 	}
 
 	/* get all active nodes attached to upstream, excluding self */
