@@ -73,7 +73,6 @@ static void start_monitoring(void);
 #ifndef WIN32
 static void setup_event_handlers(void);
 static void handle_sighup(SIGNAL_ARGS);
-static void handle_sigint(SIGNAL_ARGS);
 #endif
 
 int			calculate_elapsed(instr_time start_time);
@@ -255,6 +254,8 @@ main(int argc, char **argv)
 		strncpy(config_file_options.log_level, cli_log_level, MAXLEN);
 	}
 
+	log_notice(_("repmgrd (repmgr %s) starting up"), REPMGR_VERSION);
+
 	/*
 	 * -m/--monitoring-history, if provided, will override repmgr.conf's
 	 * monitoring_history; this is for backwards compatibility as it's
@@ -353,10 +354,24 @@ main(int argc, char **argv)
 	/* Retrieve record for this node from the local database */
 	record_status = get_node_record(local_conn, config_file_options.node_id, &local_node_info);
 
+	/*
+	 * Terminate if we can't find the local node record. This is a
+	 * "fix-the-config" situation, not a lot else we can do.
+	 */
+
 	if (record_status != RECORD_FOUND)
 	{
 		log_error(_("no metadata record found for this node - terminating"));
-		log_hint(_("check that 'repmgr (primary|standby) register' was executed for this node"));
+
+		switch (config_file_options.replication_type)
+		{
+			case REPLICATION_TYPE_PHYSICAL:
+				log_hint(_("check that 'repmgr (primary|standby) register' was executed for this node"));
+				break;
+			case REPLICATION_TYPE_BDR:
+				log_hint(_("check that 'repmgr bdr register' was executed for this node"));
+				break;
+		}
 
 		PQfinish(local_conn);
 		terminate(ERR_BAD_CONFIG);
@@ -440,6 +455,7 @@ start_monitoring(void)
 				break;
 			case WITNESS:
 				monitor_streaming_witness();
+				break;
 			case BDR:
 				monitor_bdr();
 				return;
@@ -612,11 +628,6 @@ check_and_create_pid_file(const char *pid_file)
 
 
 #ifndef WIN32
-static void
-handle_sigint(SIGNAL_ARGS)
-{
-	terminate(SUCCESS);
-}
 
 /* SIGHUP: set flag to re-read config file at next convenient time */
 static void
@@ -629,8 +640,23 @@ static void
 setup_event_handlers(void)
 {
 	pqsignal(SIGHUP, handle_sighup);
-	pqsignal(SIGINT, handle_sigint);
-	pqsignal(SIGTERM, handle_sigint);
+
+	/*
+	 * we want to be able to write a "repmgrd_shutdown" event, so delegate
+	 * signal handling to the respective replication type handler, as it
+	 * will know best which database connection to use
+	 */
+	switch (config_file_options.replication_type)
+	{
+		case REPLICATION_TYPE_BDR:
+			pqsignal(SIGINT, handle_sigint_bdr);
+			pqsignal(SIGTERM, handle_sigint_bdr);
+			break;
+		case REPLICATION_TYPE_PHYSICAL:
+			pqsignal(SIGINT, handle_sigint_physical);
+			pqsignal(SIGTERM, handle_sigint_physical);
+			break;
+	}
 }
 #endif
 
